==> tcap-1.1.tar.gz: tcap-1.1/src/prio.cpp <==
#include <arpa/inet.h>
#include <net/ethernet.h>
#include <netdb.h>
#include <netinet/ether.h>
#include <netinet/if_ether.h>
#include <netinet/ip.h>
#include <pthread.h>
#include <sys/time.h>
#include <unistd.h>

#include <iostream>
#include <map>
#include <list>


extern "C" {
#include <pcap.h>
}

using namespace std;

#include "ip.h"
#include "dns.h"
#include "prio.h"

ostream& operator<<(ostream &os, struct timeval tv) 
{
	struct tm *ti = localtime(&tv.tv_sec);

	os << ti->tm_hour << ":" << ti->tm_min << ":" << ti->tm_sec << "." << tv.tv_usec;
	return os;
}


CRecord::CRecord(const struct my_ip *ip, int volume, const char *llSrc, const char *llDst)
			: ip_src_m(ip->ip_src), srcSerial_m(-1), dstSerial_m(-1), ip_dst_m(ip->ip_dst),
					ip_proto_m(ip->ip_p), volume_m(volume), packets_m(1)  
{
	port_src_m =	ntohs(ip->port_src);
	port_dst_m =	ntohs(ip->port_dst);
	memset(&lastUpdate_m, 0, sizeof (lastUpdate_m));
	memset(&lastUpdate_m, 0, sizeof (firstUpdate_m));
	if (llSrc != 0) {
		llSrc_m = new char[strlen(llSrc) + 1];
		strcpy(llSrc_m, llSrc);
	} else {
		llSrc_m = 0;
	}
	if (llDst != 0) {
		llDst_m = new char[strlen(llDst) + 1];
		strcpy(llDst_m, llDst);
	} else {
		llDst_m = 0;
	}
}

CRecord::~CRecord()
{
	if (llSrc_m != 0)
		delete [] llSrc_m;
	if (llDst_m != 0)
		delete [] llDst_m;
}

ostream& operator<<(ostream &os, const CRecord *cr) {
	os << "src " << inet_ntoa(cr->ip_src_m)
		<< " ser " << cr->srcSerial_m
		<< " dst " << inet_ntoa(cr->ip_dst_m)
		<< " ser " << cr->dstSerial_m
		<< " sprt " << cr->port_src_m
		<< " dprt " << cr->port_dst_m
		<< " prot " << (unsigned int)cr->ip_proto_m
		<< " vol " << cr->volume_m;
	if (cr->packets_m == 1) {
		os << " " << cr->lastUpdate_m
			<< endl;
	} else {
		if (cr->lastUpdate_m.tv_sec != cr->firstUpdate_m.tv_sec)
			os << " ovr " << cr->lastUpdate_m.tv_sec - cr->firstUpdate_m.tv_sec;
		else
			os << " ovr " << cr->lastUpdate_m.tv_usec - cr->firstUpdate_m.tv_usec << " usec ";
		os << " paks " << cr->packets_m
			<< " " << cr->firstUpdate_m
			<< " " << cr->lastUpdate_m
			<< endl;
	}
	return os;
}


RTable::RTable(int SleepTime, int dataLinkType, DnsCache *dnsCache, int age, char *dev, pcap_t *descr)
			: SleepTime_m(SleepTime), dataLinkType_m(dataLinkType),
				dnsCache_m(dnsCache), age_m(age), interfaceName_m(dev), descr_m(descr) {
	pthread_mutex_init(&mutex_m, NULL);
}

void	RTable::Add(CRecord *cr, struct timeval when) {

	pthread_mutex_lock(&mutex_m);
	CRecordMapIter		crmi = crm.find(cr);
	CRecord	*ncr;
	
	if (crmi == crm.end()) {
		ncr = new CRecord(*cr);	// TODO: make proper constructor
		// put this new record to the front of the list
		crl_m.push_front(ncr);
		// save the iterator into the 'second' of the map
		crm[ncr] = crl_m.begin();
		ncr->firstUpdate_m = when;
	} else {
		CRecordListIter crli = (*crmi).second;
		ncr = (*crli);
		ncr->volume_m += cr->volume_m;
		ncr->packets_m++;
		crl_m.erase(crli);
		crl_m.push_front(ncr);
		// iterator is still there and should be valid
		(*crmi).second = crl_m.begin();
		// move this node to the head
	}
	ncr->lastUpdate_m = when;
	pthread_mutex_unlock(&mutex_m);
	
}

ostream& operator<<(ostream &os, RTable& rtt) {
		pthread_mutex_lock(&rtt.mutex_m);
		for (CRecordListIter iter = rtt.crl_m.begin(); iter != rtt.crl_m.end(); iter++) {
			os << *iter << endl;
		}
		pthread_mutex_unlock(&rtt.mutex_m);
		return os;
}


void	RTable::MakeListOfFlushItems(CRecordList *ulist, struct timeval *since) {
	pthread_mutex_lock(&mutex_m);
	for (CRecordListIter iter = crl_m.begin(); iter != crl_m.end();) {
		CRecord *cr = (*iter++); // increment iterator first as we delete this node
		// TODO: add u_sec
		// add exit when too far back
		if (cr->lastUpdate_m.tv_sec < since->tv_sec) {
			CRecordMapIter		crmi = crm.find(cr);
			if (crmi == crm.end()) {
				cerr << "shit: record was in list but not in the map!" << endl;
				exit(1);
			}
			crl_m.erase((*crmi).second); // remove from list
			crm.erase(crmi);			// remove from map
			ulist->push_back(cr);	// add to the outgoing list
		}
	}
	pthread_mutex_unlock(&mutex_m);
}

==> tcap-1.2.tar.gz: tcap-1.2/src/prio.cpp <==
#include <arpa/inet.h>
#include <net/ethernet.h>
#include <netdb.h>
#include <netinet/ether.h>
#include <netinet/if_ether.h>
#include <netinet/ip.h>
#include <pthread.h>
#include <sys/time.h>
#include <unistd.h>

#include <iostream>
#include <map>
#include <list>


extern "C" {
#include <pcap.h>
}

using namespace std;

#include "ip.h"
#include "dns.h"
#include "prio.h"

ostream& operator<<(ostream &os, struct timeval tv) 
{
	struct tm *ti = localtime(&tv.tv_sec);

	os << ti->tm_hour << ":" << ti->tm_min << ":" << ti->tm_sec << "." << tv.tv_usec;
	return os;
}


CRecord::CRecord(const struct my_ip *ip, int volume, const char *llSrc, const char *llDst)
			: ip_src_m(ip->ip_src), srcSerial_m(-1), dstSerial_m(-1), ip_dst_m(ip->ip_dst),
					ip_proto_m(ip->ip_p), volume_m(volume), packets_m(1)  
{
	port_src_m =	ntohs(ip->port_src);
	port_dst_m =	ntohs(ip->port_dst);
	memset(&lastUpdate_m, 0, sizeof (lastUpdate_m));
	memset(&lastUpdate_m, 0, sizeof (firstUpdate_m));
	if (llSrc != 0) {
		llSrc_m = new char[strlen(llSrc) + 1];
		strcpy(llSrc_m, llSrc);
	} else {
		llSrc_m = 0;
	}
	if (llDst != 0) {
		llDst_m = new char[strlen(llDst) + 1];
		strcpy(llDst_m, llDst);
	} else {
		llDst_m = 0;
	}
}

CRecord::~CRecord()
{
	if (llSrc_m != 0)
		delete [] llSrc_m;
	if (llDst_m != 0)
		delete [] llDst_m;
}

ostream& operator<<(ostream &os, const CRecord *cr) {
	os << "src " << inet_ntoa(cr->ip_src_m)
		<< " ser " << cr->srcSerial_m
		<< " dst " << inet_ntoa(cr->ip_dst_m)
		<< " ser " << cr->dstSerial_m
		<< " sprt " << cr->port_src_m
		<< " dprt " << cr->port_dst_m
		<< " prot " << (unsigned int)cr->ip_proto_m
		<< " vol " << cr->volume_m;
	if (cr->packets_m == 1) {
		os << " " << cr->lastUpdate_m
			<< endl;
	} else {
		if (cr->lastUpdate_m.tv_sec != cr->firstUpdate_m.tv_sec)
			os << " ovr " << cr->lastUpdate_m.tv_sec - cr->firstUpdate_m.tv_sec;
		else
			os << " ovr " << cr->lastUpdate_m.tv_usec - cr->firstUpdate_m.tv_usec << " usec ";
		os << " paks " << cr->packets_m
			<< " " << cr->firstUpdate_m
			<< " " << cr->lastUpdate_m
			<< endl;
	}
	return os;
}


RTable::RTable(int SleepTime, DnsCache *dnsCache, int age, char *dev)
			: SleepTime_m(SleepTime), dnsCache_m(dnsCache), age_m(age), interfaceName_m(dev) {
	pthread_mutex_init(&mutex_m, NULL);
	descr_m = 0;
	dataLinkType_m = 0;
}

void	RTable::Add(CRecord *cr, struct timeval when) {

	pthread_mutex_lock(&mutex_m);
	CRecordMapIter		crmi = crm.find(cr);
	CRecord	*ncr;
	
	if (crmi == crm.end()) {
		ncr = new CRecord(*cr);	// TODO: make proper constructor
		// put this new record to the front of the list
		crl_m.push_front(ncr);
		// save the iterator into the 'second' of the map
		crm[ncr] = crl_m.begin();
		ncr->firstUpdate_m = when;
	} else {
		CRecordListIter crli = (*crmi).second;
		ncr = (*crli);
		ncr->volume_m += cr->volume_m;
		ncr->packets_m++;
		crl_m.erase(crli);
		crl_m.push_front(ncr);
		// iterator is still there and should be valid
		(*crmi).second = crl_m.begin();
		// move this node to the head
	}
	ncr->lastUpdate_m = when;
	pthread_mutex_unlock(&mutex_m);
	
}

ostream& operator<<(ostream &os, RTable& rtt) {
		pthread_mutex_lock(&rtt.mutex_m);
		for (CRecordListIter iter = rtt.crl_m.begin(); iter != rtt.crl_m.end(); iter++) {
			os << *iter << endl;
		}
		pthread_mutex_unlock(&rtt.mutex_m);
		return os;
}


void	RTable::MakeListOfFlushItems(CRecordList *ulist, struct timeval *since) {
	pthread_mutex_lock(&mutex_m);
	for (CRecordListIter iter = crl_m.begin(); iter != crl_m.end();) {
		CRecord *cr = (*iter++); // increment iterator first as we delete this node
		// TODO: add u_sec
		// add exit when too far back
		if (cr->lastUpdate_m.tv_sec < since->tv_sec) {
			CRecordMapIter		crmi = crm.find(cr);
			if (crmi == crm.end()) {
				cerr << "shit: record was in list but not in the map!" << endl;
				exit(1);
			}
			crl_m.erase((*crmi).second); // remove from list
			crm.erase(crmi);			// remove from map
			ulist->push_back(cr);	// add to the outgoing list
		}
	}
	pthread_mutex_unlock(&mutex_m);
}


void	RTable::Set_tcapInfo(pcap_t *descr) {
	descr_m = descr;
	dataLinkType_m = pcap_datalink(descr);
}
